@@ -1,21 +1,50 @@
 import os
 import sys
+import time
 import shutil
+import ctypes
 
 from twine.settings import Settings as twine_Settings
 from twine.commands.upload import upload as twine_upload
 
 join = os.path.join
+bat_name = 'uploadtool_build_source.bat'
 
+def Command(info, cmd, params=''):
+	ctypes.windll.shell32.ShellExecuteW(None, "runas", cmd, params, None, 0)
 
-def UploadByTwine(info):
+def UploadByTwine(info, timeout=20000):
 	"""直接调用twine的cmd貌似不起效果."""
+	def _valid_dist_files(_dist_dir, _timeout):
+		def _is_files_ready():
+			tgz_flag, whl_flag = False, False
+			for _name in os.listdir(_dist_dir):
+				if _name[-3:].lower() == '.gz':
+					tgz_flag = True
+				if _name[-4:].lower() == '.whl':
+					whl_flag = True
+			return tgz_flag and whl_flag
+		
+		def _rec_for_timeout(_left_time):
+			time.sleep(0.5)
+			_left_time -= 500
+			if (_left_time <= 0):
+				raise Exception("[TimeOutError]: Failed to wait files ready.")
+			return _left_time
+		
+		if not os.path.exists(_dist_dir) or not _is_files_ready():
+			print("\n\nWaiting... (if wait too long, please ctrl+c to finish it.)\n")
+			while not os.path.exists(_dist_dir):
+				_timeout = _rec_for_timeout(_timeout)
+			while not _is_files_ready():
+				_timeout = _rec_for_timeout(_timeout)
+			
+
 	setting = twine_Settings(username=info.count, password=info.password)
 	
 	# 提取dist下所有文件(.tgz and .whl)
 	aim_path, dist_files = join(info.path, 'dist'), []
-	if not os.path.exists(aim_path) or not os.path.isdir(aim_path):
-		raise Exception("\n\n[Error]: Can not find dist dir. Please check whether you built it.")
+	_valid_dist_files(aim_path, timeout)
 	for _name in os.listdir(aim_path):
 		_path = join(info.path, 'dist', _name)
 		if os.path.isfile(_path):
@@ -30,9 +59,10 @@ class UploadInfo:
 	记录用于上传使用的信息
 	Record the information used for uploading
 	"""
-	def __init__(self, pkg_name, count=None, password=None, setup_py_dir=''):
+	def __init__(self, pkg_name, count=None, password=None, show_cmd=True):
 		self.name = pkg_name
-		self.path = setup_py_dir
+		self.path = os.getcwd()
+		self.show = show_cmd
 		self.count, self.password = count, password
 		self.itp = os.path.dirname(sys.executable)
 		self.pkg_path = join(self.itp, "Lib", "site-packages")
@@ -61,9 +91,14 @@ class UploadTool(UploadInfo):
 		self.CopyFrom()
 		print("Finish Source Update.")
 	def BuildSetup(self):
-		os.system("python setup.py bdist_wheel --universal && python setup.py sdist")
+		setup_path = join(self.path, 'setup.py')
+		bat_path = join(self.path, bat_name)
+		f = open(bat_path, 'w')
+		f.write("python {} bdist_wheel\n".format(setup_path))
+		f.write("python {} sdist".format(setup_path))
+		f.close()
+		Command(self, bat_path)
 		UploadByTwine(self)
-		os.system("pause")
 	
 	def Upload(self):
 		"""
